@@ -5,6 +5,7 @@ use std::sync::Arc;
 use syndicate::actor::*;
 use syndicate::during::entity;
 use syndicate::enclose;
+use syndicate::preserves::rec;
 use syndicate::schemas::dataspace::Observe;
 use syndicate::schemas::service;
 use syndicate::value::NestedValue;
@@ -16,10 +17,10 @@ use crate::schemas::internal_services;
 use syndicate_macros::during;
 
 pub fn boot(t: &mut Activation, ds: Arc<Cap>) {
-    t.spawn(syndicate::name!("dependencies"), move |t| {
+    t.spawn(Some(AnyValue::symbol("dependencies_listener")), move |t| {
         Ok(during!(t, ds, language(), <require-service $spec>, |t: &mut Activation| {
             tracing::debug!(?spec, "tracking dependencies");
-            t.spawn_link(syndicate::name!(parent: None, "dependencies", spec = ?spec),
+            t.spawn_link(Some(rec![AnyValue::symbol("dependencies"), language().unparse(&spec)]),
                          enclose!((ds) |t| run(t, ds, spec)));
             Ok(())
         }))
@@ -53,8 +54,8 @@ fn run(t: &mut Activation, ds: Arc<Cap>, service_name: AnyValue) -> ActorResult
     }
 
     let obstacle_count = t.named_field("obstacle_count", 1isize);
-    t.dataflow(enclose!((obstacle_count) move |t| {
-        tracing::trace!(obstacle_count = ?t.get(&obstacle_count));
+    t.dataflow(enclose!((service_name, obstacle_count) move |t| {
+        tracing::trace!(?service_name, obstacle_count = ?t.get(&obstacle_count));
         Ok(())
     }))?;
 
@@ -73,24 +74,25 @@ fn run(t: &mut Activation, ds: Arc<Cap>, service_name: AnyValue) -> ActorResult
         })
     })?;
 
+    let depender = service_name.clone();
     enclose!((ds, obstacle_count) during!(
-        t, ds, language(), <depends-on #(&service_name) $dependee>,
-        enclose!((ds, obstacle_count) move |t: &mut Activation| {
+        t, ds, language(), <depends-on #(&depender) $dependee>,
+        enclose!((service_name, ds, obstacle_count) move |t: &mut Activation| {
             if let Ok(dependee) = language().parse::<service::ServiceState>(&dependee) {
-                tracing::trace!(on = ?dependee, "new dependency");
+                tracing::trace!(?service_name, ?dependee, "new dependency");
                 ds.assert(t, language(), &service::RequireService {
                     service_name: dependee.service_name,
                 });
             } else {
-                tracing::warn!(on = ?dependee, "cannot deduce dependee service name");
+                tracing::warn!(?service_name, ?dependee, "cannot deduce dependee service name");
             }
 
             counter::adjust(t, &obstacle_count, 1);
 
             let d = &dependee.clone();
             during!(t, ds, language(), #d, enclose!(
-                (obstacle_count, dependee) move |t: &mut Activation| {
-                    tracing::trace!(on = ?dependee, "dependency satisfied");
+                (service_name, obstacle_count, dependee) move |t: &mut Activation| {
+                    tracing::trace!(?service_name, ?dependee, "dependency satisfied");
                     counter::adjust(t, &obstacle_count, -1);
                     Ok(())
                 }));
